@@ -8,16 +8,26 @@ use JSON::XS;
 use File::Temp qw(tempfile);
 
 use Compress::Zlib;
-use MIME::Base64 qw(decode_base64url);
+use MIME::Base64;
+use URI::Escape;
 
-use Test::More tests => 3;
+
+use Test::More tests => 5;
 
 BEGIN { use_ok( 'JSON::Builder' ) }
 
 my $json = JSON::XS->new()->utf8(1)->ascii(1);
 
 
-sub foo {
+sub read_fh {
+	my ($fh) = @_;
+	$fh->flush();
+	$fh->seek(0,0);
+	join "", <$fh>;
+}
+
+
+sub build {
 	my ($fh, $builder) = @_;
 
 	my $fv = $builder->val( { a => 'b', c => 'd' } );
@@ -43,9 +53,6 @@ sub foo {
 
 	$builder->encode(\%d);
 
-	$fh->flush();
-	$fh->seek(0,0);
-	join "", <$fh>;
 }
 
 
@@ -65,6 +72,7 @@ my $j = {
 };
 
 
+# Simple
 {
 	my ($fh) = tempfile(UNLINK => 1);
 
@@ -74,19 +82,52 @@ my $j = {
 		read_in => 1000*57
 	);
 
-	my $r = foo($fh, $builder);
+	build($fh, $builder);
+
+	my $r = read_fh($fh);
 	is_deeply($json->decode($r), $j, "Simple");
 }
+
+
+# Plain
 {
+	my ($fh)       = tempfile(UNLINK => 1);
+	my ($fh_plain) = tempfile(UNLINK => 1);
+
+	my $builder = JSON::Builder::Compress->new(
+		json     => $json,
+		fh       => $fh,
+		read_in  => 1000*57,
+		fh_plain => $fh_plain, 
+	);
+
+	build($fh, $builder);
+
+	my $r = read_fh($fh);
+	my $rj = uncompress(MIME::Base64::decode_base64url($r));
+	is_deeply($json->decode($rj), $j, "Compress and Base64");
+
+	my $r_plain = read_fh($fh_plain);
+	is_deeply($json->decode($rj), $json->decode($r_plain), "Plain");
+}
+
+
+# encode_sub
+{
+
 	my ($fh) = tempfile(UNLINK => 1);
 
 	my $builder = JSON::Builder::Compress->new(
-		json    => $json,
-		fh      => $fh,
-		read_in => 1000*57
+		json     => $json,
+		fh       => $fh,
+		read_in  => 1000*57,
+		encode_sub        => sub { uri_escape(encode_base64($_[0], "")) },
+		encode_chunk_size => 57,
 	);
 
-	my $r = foo($fh, $builder);
-	my $rj = uncompress(decode_base64url($r));
-	is_deeply($json->decode($rj), $j, "Compress, Base64");
+	build($fh, $builder);
+
+	my $r = read_fh($fh);
+	my $rj = uncompress(decode_base64(uri_unescape($r)));
+	is_deeply($json->decode($rj), $j, "Compress and encode_sub");
 }
